@@ -3,11 +3,12 @@ use 5.010001;
 use strict;
 use warnings;
 
+use Class::Accessor::Inherited::XS::Compat qw/mk_type_accessors mk_inherited_accessors mk_class_accessors mk_varclass_accessors mk_object_accessors/;
+
 use Carp ();
 
-our $VERSION = '0.28_004';
+our $VERSION = '0.35_01';
 our $PREFIX  = '__cag_';
-our $NEED_COMPAT = 0;
 
 require XSLoader;
 XSLoader::load('Class::Accessor::Inherited::XS', $VERSION);
@@ -33,12 +34,10 @@ sub import {
 
     my %opts = ref($_[0]) eq 'HASH' ? %{ $_[0] } : @_;
     my $class = delete $opts{package} // caller;
-    local $NEED_COMPAT = 1 if delete $opts{compability};
 
     for my $type (keys %opts) {
         my $accessors = $opts{$type};
-        my $installer = _type_installer($type);
-        my $clone_arg = $REGISTERED_TYPES->{$type}{clone_arg};
+        my ($installer, $clone_arg) = $pkg->_type_installer($type);
 
         if (ref($accessors) eq 'HASH') {
             $installer->($class, $_, $accessors->{$_}) for keys %$accessors;
@@ -55,42 +54,6 @@ sub import {
     }
 }
 
-sub mk_inherited_accessors {
-    my $class = shift;
-    mk_type_accessors($class, 'inherited', @_);
-}
-
-sub mk_class_accessors {
-    my $class = shift;
-    mk_type_accessors($class, 'class', @_);
-}
-
-sub mk_varclass_accessors {
-    my $class = shift;
-    mk_type_accessors($class, 'varclass', @_);
-}
-
-sub mk_object_accessors {
-    my $class = shift;
-    mk_type_accessors($class, 'object', @_);
-}
-
-sub mk_type_accessors {
-    my ($class, $type) = (shift, shift);
-
-    my $installer = _type_installer($type);
-    my $clone_arg = $REGISTERED_TYPES->{$type}{clone_arg};
-
-    for my $entry (@_) {
-        if (ref($entry) eq 'ARRAY') {
-            $installer->($class, @$entry);
-
-        } else {
-            $installer->($class, $entry, $clone_arg && $entry);
-        }
-    }
-}
-
 sub register_types {
     register_type(shift, shift) while scalar @_;
 }
@@ -108,12 +71,14 @@ sub register_type {
         $args->{installer} = sub {
             my ($class, $name, $field) = @_;
             install_inherited_cb_accessor(
-                "${class}::${name}", $field, $PREFIX.$field, $args->{read_cb} // $args->{on_read}, $args->{write_cb} // $args->{on_write}
+                "${class}::${name}", $field, $PREFIX.$field,
+                $args->{read_cb} // $args->{on_read}, $args->{write_cb} // $args->{on_write},
+                $args->{opts} // 0,
             );
         };
     }
 
-    $args->{clone_arg} = 1 unless exists $args->{clone_arg};
+    $args->{clone_arg} = 1 unless exists $args->{clone_arg}; # for cb-types
     $REGISTERED_TYPES->{$type} = $args;
 }
 
@@ -122,22 +87,22 @@ sub register_type {
 #
 
 sub _type_installer {
-    my $type = shift;
+    my (undef, $type) = @_;
 
     my $type_info = $REGISTERED_TYPES->{$type} or Carp::confess("Don't know how to install '$type' accessors");
-    return $type_info->{installer};
+    return ($type_info->{installer}, $type_info->{clone_arg});
 }
 
 sub _mk_inherited_accessor {
     my ($class, $name, $field, $flags) = @_;
 
-    install_inherited_accessor("${class}::${name}", $field, $PREFIX.$field, $flags | ($NEED_COMPAT ? 2 : 0));
+    install_inherited_accessor("${class}::${name}", $field, $PREFIX.$field, $flags);
 }
 
 sub _mk_class_accessor {
-    my ($class, $name, $default, $is_varclass, $is_readonly) = @_;
+    my ($class, $name, $default, $is_varclass, $is_readonly, $is_weak) = @_;
 
-    install_class_accessor("${class}::${name}", $default, $is_varclass, $is_readonly);
+    install_class_accessor("${class}::${name}", $default, $is_varclass, ($is_readonly ? 1 : 0) | ($is_weak ? 2 : 0));
 }
 
 sub _mk_object_accessor {
@@ -181,9 +146,9 @@ Class::Accessor::Inherited::XS - Fast XS inherited, object and class accessors
       getters   => ['foo'],             # alias for 'object_ro'
   };
 
-  # or at run time, in a Class::Accessor::Grouped-like fashion
-  # this is unrecommended style and it provides very limited interface
-  use parent 'Class::Accessor::Inherited::XS';
+  # Or if you prefer a Class::Accessor::Grouped-like interface, you can do it
+  # at run time. Note that this is not recommended and provides limited feature set.
+  use parent 'Class::Accessor::Inherited::XS::Compat';
 
   __PACKAGE__->mk_inherited_accessors('foo', [bar => 'bar_key']);
   __PACKAGE__->mk_class_accessors('foo');
@@ -283,9 +248,6 @@ class_caix       31300776/s          13286%   2960%   2303%        391%
         inherited_cb => ['bar'],
     };
 
-    #or in a Class::Accessor::Grouped-like fashion
-    __PACKAGE__->mk_type_accessors(inherited_cb => 'foo', 'bar');
-
 You can register new inherited accessor types with associated read/write callbacks. Unlike
 L<Class::Accessor::Grouped>, only a single callback can be set for a type, without per-class
 B<get_$type>/B<set_$type> lookups. You can omit either B<on_read> or B<on_write> if you don't
@@ -329,7 +291,7 @@ and $__PACKAGE__::accessor variable. While the stored value is still accessible
 
 Copyright (C) 2009 by Vladimir Timofeev
 
-Copyright (C) 2014-2015 by Sergey Aleynikov
+Copyright (C) 2014-2018 by Sergey Aleynikov
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.10.1 or,
