@@ -65,4 +65,19 @@ static shared_keys* CAIXS_install_accessor(pTHX_ int val, SV* full_name) {
     return CAIXS_payload_init<type>(aTHX_ cv);
 }};
 
+/*
+template <AccessorOpts opts>
+static void CAIXS_export_opts(pTHX_ HV* stash) {
+//    newCONSTSUB(stash, 
+
+    return CImpl<type, (AccessorOpts)(opts-1)>::CAIXS_install_accessor(aTHX_ val, full_name);
+}};
+
+template <AccessorType type>
+static void CAIXS_install_accessor(pTHX_ HV* stash) {
+    CV* cv = CAIXS_install_cv<type, None>(aTHX_ full_name);
+    return CAIXS_payload_init<type>(aTHX_ cv);
+}};
+*/
+
 #endif /* __INHERITED_XS_INSTALLER_H_ */
